package com.glxp.udidl.admin.controller.device;

import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dao.udid.DevicedownloadMapper;
import com.glxp.udidl.admin.entity.udid.*;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.udid.*;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.res.udid.ProgressResponse;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 未转换国家库查询与下载
 */
@Api(tags = "未转换国家库查询与下载接口")
@RestController
public class DeviceController {

    @Resource
    private DeviceService deviceService;
    @Resource
    private DevicedownloadMapper devicedownloadMapper;
    @Resource
    CompanyService companyService;
    @Resource
    ProductInfoService productInfoService;
    @Resource
    UdiCompanyService udiCompanyService;

    @ApiOperation(value = "查询医疗器械信息", response = DeviceEntity.class)
    @AuthRuleAnnotation("warehouse/device/list")
    @GetMapping("warehouse/device/list")
    public BaseResponse listDevices(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        boolean isScan = false;
        if (key != null) {
            key = FilterUdiUtils.getDiStr(key);
            deviceListRequest.setKey(key);
            isScan = true;
        }
        List<DeviceEntity> medicalResonseList = deviceService.listDevices(deviceListRequest);
        long total = 0;
        PageInfo<DeviceEntity> pageInfo = new PageInfo<>(medicalResonseList);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        List<Devicedownload> devicedownloads = devicedownloadMapper.selectByExample(new DevicedownloadExample());
        Devicedownload devicedownload;
        if (devicedownloads != null && devicedownloads.size() > 0) {
            devicedownload = devicedownloads.get(0);
            total = devicedownload.getAlltotal() - devicedownload.getAllindex();
        }
        if (isScan) {
            total = 20;
            pageSimpleResponse.setTotal(total);
        } else
            pageSimpleResponse.setTotal(total * 10);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @ApiOperation(value = "根据关键字查询医疗器械信息", response = DeviceEntity.class)
    @AuthRuleAnnotation("warehouse/device/search")
    @GetMapping("warehouse/device/search")
    public BaseResponse searchByKey(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        boolean isScan = false;
        if (key != null) {
            key = FilterUdiUtils.getDiStr(key);
            deviceListRequest.setKey(key);
            isScan = true;
        } else {

        }
        DeviceSearchRequest deviceSearchRequest = new DeviceSearchRequest();
        deviceSearchRequest.setZxxsdycpbs(key);
        deviceSearchRequest.setPage(1);
        deviceSearchRequest.setLimit(50);
        List<DeviceEntity> devices = deviceService.search(deviceSearchRequest);
        if (key == null || key.equals("") || devices == null || devices.size() == 0) {
            devices = new ArrayList<>();
            devices.add(new DeviceEntity());
        }
        long total = 1;
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(devices.get(devices.size() - 1));
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @ApiOperation(value = "根据DI查询医疗器械信息", response = DeviceEntity.class)
    @AuthRuleAnnotation("warehouse/device/searchbydi")
    @GetMapping("warehouse/device/searchbydi")
    public BaseResponse searchByDi(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();
        Device device = deviceService.searchByZxxscpbs(key);
        long total = 1;
        DeviceEntity deviceEntity = new DeviceEntity();
        if (device != null)
            BeanUtils.copyProperties(device, deviceEntity);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(deviceEntity);
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @GetMapping("warehouse/device/unionSearch")
    public BaseResponse search(@Valid DeviceSearchRequest deviceSearchRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<DeviceEntity> medicalResonseList = deviceService.search(deviceSearchRequest);
        long total = deviceService.seatchTotal(deviceSearchRequest);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @GetMapping("warehouse/device/combineSearch")
    public BaseResponse combineSearch(@Valid DeviceSearchRequest deviceSearchRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<DeviceEntity> medicalResonseList = deviceService.search(deviceSearchRequest);

        long total = 0;
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        List<Devicedownload> devicedownloads = devicedownloadMapper.selectByExample(new DevicedownloadExample());
        Devicedownload devicedownload;
        if (devicedownloads != null && devicedownloads.size() > 0) {
            devicedownload = devicedownloads.get(0);
            total = devicedownload.getAlltotal() - devicedownload.getAllindex();
        }


        pageSimpleResponse.setTotal(total * 10);
        pageSimpleResponse.setList(medicalResonseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @PostMapping("warehouse/device/loadsingle")
    public BaseResponse downLoadDiii(@RequestBody DeviceDiRequest deviceRequest) {
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyService, udiCompanyService);
        if (deviceRequest.getPrimaryDeviceId() == null) {
            return ResultVOUtils.error(999, "最小销售单元标识不能为空");
        }
        new Thread(() -> {

            TokenEntity token = downloadDeviceHelper.getToken();
            DownloadDiRequest downloadDiRequest1 = new DownloadDiRequest();
            downloadDiRequest1.setAccessToken(token.getAccessToken());
            downloadDiRequest1.setPrimaryDeviceId(deviceRequest.getPrimaryDeviceId());
            int result = downloadDeviceHelper.downloadBydi(downloadDiRequest1, deviceService, productInfoService);
            if (result == -1) {
                return;// ResultVOUtils.error(505, "下载出错");
            }

        }).start();
        return ResultVOUtils.success("开始下载");
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @PostMapping("warehouse/device/downLoadHisory")
    public BaseResponse downLoadHisory(@RequestBody DeviceDiRequest deviceRequest) {
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyService, udiCompanyService);
        if (deviceRequest.getPrimaryDeviceId() == null) {
            return ResultVOUtils.error(999, "最小销售单元标识不能为空");
        }
        new Thread(() -> {
            String deviceRecordKey = deviceService.selectKey(deviceRequest.getPrimaryDeviceId());
            TokenEntity token = downloadDeviceHelper.getToken();
            DownloadHistoryRequest deviceHistoryRequest = new DownloadHistoryRequest();
            deviceHistoryRequest.setAccessToken(token.getAccessToken());
            deviceHistoryRequest.setCurrentPageNumber("1");
            List<DownloadHistoryRequest.DataSetBean> dataSetBeans = new ArrayList<>();
            DownloadHistoryRequest.DataSetBean keys = new DownloadHistoryRequest.DataSetBean();
            keys.setDeviceRecordKey(deviceRecordKey);
            dataSetBeans.add(keys);
            deviceHistoryRequest.setDataSet(dataSetBeans);
            int result = downloadDeviceHelper.downloadHistory(productInfoService, deviceHistoryRequest, deviceService);
            if (result == -1) {
                return;// ResultVOUtils.error(505, "下载出错");
            }

        }).start();
        return ResultVOUtils.success("开始下载");
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @PostMapping("warehouse/device/loadudid")
    public BaseResponse downLoadAll(DownloadDeviceRequest downloadDeviceRequest, BindingResult bindingResult) {
        DownloadDeviceHelper downloadDeviceHelper = new DownloadDeviceHelper(companyService, udiCompanyService);
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        new Thread(() -> {

            TokenEntity token = downloadDeviceHelper.getToken();
            int total = downloadDeviceHelper.getTotal(token.getAccessToken());
            int curpage = total;
            Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey("udid");
            if (devicedownload != null) {
                int offset = devicedownload.getAlltotal() - devicedownload.getAllindex();
                curpage = total - offset;
            } else {
                devicedownload = new Devicedownload();
                devicedownload.setId("udid");
                devicedownload.setAlltotal(100);
                devicedownload.setAllindex(99);
                devicedownloadMapper.insert(devicedownload);
            }
            devicedownload.setIsdownload(1);
            long startTime = System.currentTimeMillis();
            devicedownload.setStarttime(startTime + "");
            devicedownloadMapper.updateByPrimaryKey(devicedownload);
            Devicedownload stop = devicedownloadMapper.selectByPrimaryKey("udid");

            while (curpage > 0 && stop.getIsdownload() == 1 && stop.getStarttime().equals(startTime + "")) {

                DeviceRequest deviceRequest = new DeviceRequest();
                deviceRequest.setAccessToken(token.getAccessToken());
                deviceRequest.setRequestType(3 + "");
                deviceRequest.setCurrentPageNumber(curpage + "");
                int result = downloadDeviceHelper.downloadDevices(productInfoService, deviceService, deviceRequest);
                if (result == -1) {
                    return;// ResultVOUtils.error(505, "下载出错");
                }
                curpage--;
                Devicedownload devicedownload1 = new Devicedownload();
                devicedownload1.setAlltotal(total);
                devicedownload1.setAllindex(curpage);
                devicedownload1.setId("udid");
                devicedownloadMapper.updateProgress(devicedownload1);
                stop = devicedownloadMapper.selectByPrimaryKey("udid");
            }
        }).start();
        return ResultVOUtils.success("开始下载");
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @GetMapping("warehouse/device/stop")
    public BaseResponse stopDown() {
        Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey("udid");
        devicedownload.setIsdownload(2);
        devicedownloadMapper.updateByPrimaryKey(devicedownload);
        return ResultVOUtils.success("停止成功");
    }

    @Deprecated
    @ApiOperation(value = "", hidden = true)
    @GetMapping("warehouse/device/progress")
    public BaseResponse getProgress() {
        Devicedownload devicedownload = devicedownloadMapper.selectByPrimaryKey("udid");
        ProgressResponse response = new ProgressResponse();
        if (devicedownload != null) {
            double index = devicedownload.getAlltotal() - devicedownload.getAllindex();
            double total = devicedownload.getAlltotal();
            double progress = 0;
            if (total > 0) {
                progress = index / total;
            }
            response.setTotal(total * 10);
            response.setIndex(index * 10);
            response.setProgress(progress * 100);
        } else {
            response.setTotal(100);
            response.setIndex(1);
            response.setProgress(1);
        }

        return ResultVOUtils.success(response);
    }


}
