package com.glxp.udidl.admin.controller.device;

import com.alibaba.fastjson.JSONObject;
import com.glxp.udidl.admin.dao.udi.ProductInfoMapper;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udi.UdiCompanyEntity;
import com.glxp.udidl.admin.entity.udid.Contactlist;
import com.glxp.udidl.admin.entity.udid.Device;
import com.glxp.udidl.admin.req.ListPageRequest;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.UdiCompanyRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.dataSync.DeviceSyncService;
import com.glxp.udidl.admin.service.dataSync.UdplatDownloadService;
import com.glxp.udidl.admin.service.dataSync.UdplatSyncService;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.thread.AsyncDownloadTask;
import com.glxp.udidl.admin.thread.UdiTransferUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@ApiIgnore
@RestController
public class TestController {

    private static final Logger logger = LoggerFactory.getLogger(TestController.class);

    @Resource
    private DeviceService deviceService;
    @Resource
    private ProductInfoService productInfoService;
    @Resource
    AsyncDownloadTask asyncDownloadTask;
    @Resource
    ProductInfoMapper productInfoDao;
    @Resource
    UdiCompanyService udiCompanyService;
    @Autowired
    DeviceSyncService deviceSyncService;
    @Autowired
    UdplatDownloadService udplatDownloadService;
    @Autowired
    UdplatSyncService udplatSyncService;

    @GetMapping("udidl/device/transfer")
    public void transfer() {
        List<String> keys = deviceService.findAllByZxxscpbs();
        for (String key : keys) {
            new UdiTransferUtils().transUdi(deviceService, productInfoService, key, udiCompanyService);
        }
    }

    @GetMapping("udidl/device/updateProduct")
    public BaseResponse updateProduct(ListPageRequest listPageRequest) {
        List<String> keys = productInfoDao.findAllUuids(listPageRequest);
        int index = listPageRequest.getPage();
        for (String key : keys) {
            index++;
            new UdiTransferUtils().updateUdi(deviceService, productInfoService, key);
            logger.info(listPageRequest.getPage() + "---" + listPageRequest.getLimit() + "----" + index);
        }
        return ResultVOUtils.success("总数=" + keys.size() + "\n");
    }

    @GetMapping("udidl/device/transferByPage")
    public BaseResponse transferByPage(ListPageRequest listPageRequest) {
        List<String> keys = deviceService.findAllByZxxscpbsPage(listPageRequest);
        for (String key : keys) {
            new UdiTransferUtils().transUdi(deviceService, productInfoService, key, udiCompanyService);
        }
        return ResultVOUtils.success("总数=" + keys.size() + "\n");
    }

    @GetMapping("udidl/device/transferByDi")
    public void transferByDi(@RequestParam List<String> keys) {
        if (keys != null) {
            for (String key : keys) {
                Device device = deviceService.searchByZxxscpbs(key);
                if (device != null)
                    new UdiTransferUtils().transUdi(deviceService, productInfoService, device.getUuid(), udiCompanyService);
            }
        }
    }

    //https://www.udims.com/UDI_DL_Server/udidl/device/dlBymonth?month=2021-10&page=1
    @GetMapping("udidl/device/dlBymonth")
    public void dlBymonth(String month, int page) {
        logger.info(month + "---按月开启下载");
        asyncDownloadTask.downloadByMonth(month, page);
    }

    @GetMapping("udidl/device/dlBymonthTest")
    public void dlBymonthTest(String month, int page) {

        logger.info(month + "---按月开启下载");
        asyncDownloadTask.downloadByMonthTest(month, page);
    }

    @GetMapping("/test/getDistributor")
    public String udplatTest() {
        return udplatDownloadService.getDistributor();
    }

    @GetMapping("/test/getManufactures")
    public BaseResponse getManufactures() {
        return udplatDownloadService.getAllManufactures();
    }

    @GetMapping("/test/udplat")
    public BaseResponse udplatSync() {
        udplatSyncService.downloadAndSave("manual", "");
        return ResultVOUtils.success("后台已开始下载！");
    }

    @GetMapping("/test/udplatGoods")
    public ResponseEntity<String> getGoods() {
        return udplatDownloadService.getUdplatGoods(1, 100);
    }

    @GetMapping("/test/getPurchaseOrders")
    public ResponseEntity<String> getPurchaseOrders(String orderCode) {
        return udplatDownloadService.getPurchaseOrders(orderCode);
    }

    @GetMapping("/test/getPurchaseOrderDetals")
    public ResponseEntity<String> getPurchaseOrderDetals(Long orderId) {
        return udplatDownloadService.getPurchaseOrderDetals(orderId);
    }

    @GetMapping("/test/getDistributeOrderDetails")
    public JSONObject getDistributeOrderDetails(Long orderId) {
        return udplatDownloadService.getDistributeOrderDetails(orderId);
    }

    @GetMapping("/test/transferUdiCompany")
    public BaseResponse transferUdiCompany(String nameCode) {
        //更新为最新版本
        ProductInfoFilterRequest filterRequest1 = new ProductInfoFilterRequest();
        filterRequest1.setNameCode(nameCode);
        List<ProductInfoEntity> updateEntitys = productInfoService.filterUdi(filterRequest1);
        ProductInfoEntity productInfoEntity;
        if (updateEntitys != null && updateEntitys.size() > 0) {
            //获取最高版本号
            productInfoEntity = updateEntitys.get(0);
            //更新厂家信息，
            UdiCompanyRequest udiCompanyRequest = new UdiCompanyRequest();
            udiCompanyRequest.setTyshxydm(productInfoEntity.getTyshxydm());
            List<UdiCompanyEntity> udiCompanyEntitys = udiCompanyService.filterUdiCompany(udiCompanyRequest);
            if (udiCompanyEntitys == null || udiCompanyEntitys.size() == 0) {
                UdiCompanyEntity udiCompanyEntity = new UdiCompanyEntity();
                udiCompanyEntity.setTyshxydm(productInfoEntity.getTyshxydm());
                udiCompanyEntity.setUpdateTime(new Date());
                udiCompanyEntity.setYlqxzcrbarywmc(productInfoEntity.getYlqxzcrbarywmc());
                udiCompanyEntity.setYlqxzcrbarmc(productInfoEntity.getYlqxzcrbarmc());
                List<Contactlist> contactlists = udiCompanyService.selectContactBykey(productInfoEntity.getDeviceRecordKey());
                if (contactlists != null && contactlists.size() > 0) {
                    udiCompanyEntity.setQylxrcz(contactlists.get(0).getQylxrcz());
                    udiCompanyEntity.setQylxrdh(contactlists.get(0).getQylxrdh());
                    udiCompanyEntity.setQylxryx(contactlists.get(0).getQylxryx());
                }
                udiCompanyEntity.setUpdateTime(new Date());
                udiCompanyService.insertUdiCompany(udiCompanyEntity);
                return ResultVOUtils.success(udiCompanyEntity);
            }

        }
        return ResultVOUtils.error(500, "未查询到数据");
    }

}
