package com.glxp.udidl.admin.service.inout.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.udidl.admin.dao.udi.ProductInfoMapper;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class ProductInfoServiceImpl implements ProductInfoService {

    @Resource
    ProductInfoMapper productInfoDao;

    @Override
    public List<ProductInfoEntity> filterCpmctymc(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
        PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        List<ProductInfoEntity> data = productInfoDao.filterCpmctymc(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> filterProductInfo(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }

        List<ProductInfoEntity> data = productInfoDao.filterProductInfo(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> filterUdiByTyshxydm(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByTyshxydm(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> filterUdiByCreditNo(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByCreditNo(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> filterUdiByNewest(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByNewest(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> findAllByUuid(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        List<String> uuids = productInfoDao.filterUuidByCreditNo(productInfoFilterRequest);
        if (uuids != null && uuids.size() > 0)
            return productInfoDao.selectAllByUuid(uuids);
        else
            return null;
    }

    @Override
    public List<ProductInfoEntity> filterUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        if (productInfoFilterRequest.getPage() != null) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }

        List<ProductInfoEntity> data = productInfoDao.filterUdi(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> selectByUuid(String uuid) {
        List<ProductInfoEntity> data = productInfoDao.selectByUuid(uuid);
        return data;
    }

    @Override
    public List<ProductInfoEntity> syncDlUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit() + 1;
        productInfoFilterRequest.setPage(offset);
        return productInfoDao.syncDlUdi(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> findAll(ProductInfoFilterRequest productInfoFilterRequest) {
        List<ProductInfoEntity> data = productInfoDao.filterProductInfo(productInfoFilterRequest);
        return data;
    }

    public List<ProductInfoEntity> selectByUpdateTime(String startDate, String endDate) {
        return productInfoDao.selectByUpdateTime(startDate, endDate);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean insertProductInfo(ProductInfoEntity productInfoEntity) {
        return productInfoDao.insertProductInfo(productInfoEntity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateProductByUuid(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateProductByUuid(productInfoEntity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateProductInfo(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateProductInfo(productInfoEntity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateCplx(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateCplx(productInfoEntity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteById(String id) {
        return productInfoDao.deleteById(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteAll(List<String> ids) {
        return productInfoDao.deleteAll(ids);
    }

}