package com.glxp.udidl.admin.thread;

import com.glxp.udidl.admin.dao.info.ScheduledMapper;
import com.glxp.udidl.admin.entity.info.ScheduledEntity;
import com.glxp.udidl.admin.entity.udi.CountProductEntity;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.DeviceEntity;
import com.glxp.udidl.admin.entity.udid.Devicepackage;
import com.glxp.udidl.admin.req.info.ScheduledRequest;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

//@Component
//@EnableScheduling
public class UdiTransferTask implements SchedulingConfigurer {

    final Logger logger = LoggerFactory.getLogger(DownloadTask.class);

    @Resource
    private ScheduledMapper scheduledDao;
    @Resource
    private DeviceService deviceService;
    @Resource
    private ProductInfoService productInfoService;


    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("transUdi");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();//"0 55 5 * * ?";
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }

    private void process() {
        List<String> keys = deviceService.findAllByZxxscpbs();
        for (String key : keys) {
            transUdi(key);
        }
    }

    public void transUdi(String key) {
        DeviceEntity deviceEntity = deviceService.searchById(key);
        ProductInfoEntity productInfoEntity = new ProductInfoEntity();
        productInfoEntity.setDeviceRecordKey(deviceEntity.getDevicerecordkey());
        productInfoEntity.setNameCode(deviceEntity.getZxxsdycpbs());
        productInfoEntity.setAddType(1 + "");
        productInfoEntity.setBhxjcpbm("");
        productInfoEntity.setBhxjsl(1);
        productInfoEntity.setBzcj("");
        productInfoEntity.setBhzxxsbzsl(1);
        productInfoEntity.setPackLevel(1 + "");
        productInfoEntity.setZxxsbzbhsydysl(Integer.parseInt(deviceEntity.getZxxsdyzsydydsl()));
        productInfoEntity.setUpdateTime(new Date());
        productInfoService.insertProductInfo(productInfoEntity);
        List<Devicepackage> devicepackages = deviceEntity.getDevicepackages();
        List<CountProductEntity> countProductEntities = new ArrayList<>();
        if (devicepackages != null && devicepackages.size() > 0) {
            HashMap<String, String> packRation = new HashMap<>();
            for (Devicepackage devicepackage : devicepackages) {
                CountProductEntity countProductEntity = new CountProductEntity();
                Devicepackage head = devicepackage;
                int level = 2;      //包装层级，初始为2，最小销售单元为1
                int zxbsCount = devicepackage.getBznhxyjcpbssl();  //最小销售单元数量
                int count = 1;     //计数，如果计数超过包装标识层级，就是死循环，包装标识有误
                while (!head.getBznhxyjbzcpbs().equals(deviceEntity.getZxxsdycpbs())) {
                    head = getNext(devicepackages, head);
                    if (head == null) {
                        return;
                    }
                    count++;
                    if (count > devicepackages.size()) {
                        return;
                    }
                    level++;
                    zxbsCount = zxbsCount * head.getBznhxyjcpbssl();

                }
                packRation.put(level - 1 + "", devicepackage.getBznhxyjcpbssl() + "");
                countProductEntity.setLevel(level);
                countProductEntity.setZxbsCount(zxbsCount);
                countProductEntities.add(countProductEntity);
            }
            CountProductEntity max = getMaxLevel(countProductEntities);
            packRation.put(max.getLevel() + "", max.getZxbsCount() + "");
            for (int i = 0; i < devicepackages.size(); i++) {
                Devicepackage devicepackage = devicepackages.get(i);
                CountProductEntity countProductEntity = countProductEntities.get(i);
                productInfoEntity.setNameCode(devicepackage.getBzcpbs());
                productInfoEntity.setBhxjcpbm(devicepackage.getBznhxyjbzcpbs());
                productInfoEntity.setBhxjsl(devicepackage.getBznhxyjcpbssl());
                productInfoEntity.setBzcj(devicepackage.getCpbzjb());

                productInfoEntity.setBhzxxsbzsl(countProductEntity.getZxbsCount());
                productInfoEntity.setPackLevel(countProductEntity.getLevel() + "");
                productInfoEntity.setUpdateTime(new Date());
                productInfoService.insertProductInfo(productInfoEntity);
            }
        }
    }

    public Devicepackage getNext(List<Devicepackage> devicepackages, Devicepackage devicepackage) {

        if (devicepackages != null && devicepackages.size() > 0) {
            for (int i = 0; i < devicepackages.size(); i++) {
                Devicepackage temp = devicepackages.get(i);
                if (temp.getBzcpbs().equals(devicepackage.getBznhxyjbzcpbs())) {
                    return temp;
                }
            }
        }
        return null;
    }

    public CountProductEntity getMaxLevel(List<CountProductEntity> countProductEntities) {
        int index = 0;
        int max = 0;
        for (int i = 0; i < countProductEntities.size(); i++) {
            CountProductEntity countProductEntity = countProductEntities.get(i);
            if (countProductEntity.getZxbsCount() > max) {
                max = countProductEntity.getZxbsCount();
                index = i;
            }
        }
        return countProductEntities.get(index);
    }


}
